package com.jacquesb.planetario_a;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.TimeZone;

/**  <pre>
 * Class containing all planet (orbit) data
 * and some tools (methods) to convert earth celestial (= earth equator) data to ecliptic XYZ coordinates
 * @author jacques
 *
 */
public class PlanetDataSource {
	/**  <pre>
	 * Self-test of the PlanetData 
	 * - useful if new data is added
	 * - must run without errors 
	 * @param args	unused
	 */
	public static void main(String[] args) {
		PlanetDataSource pd = new PlanetDataSource();
		System.out.println("Self-test of PlanetDataSource");
		System.out.println("=============================");
		System.out.println("PlanetData holds data for:");

		System.out.println("planet\tnOrbitX\tnOrbitY\tnOrbitZ\tnNorthPoleX\tnNorthPoleY\tNorthPoleZ");
		Enumeration<String> en = pd.planetDataTable.keys();
		while (en.hasMoreElements()) {
			PlanetDataStructure pp = pd.planetDataTable.get(en.nextElement());
			System.out.println(pp.name+"\t"+pp.orbitRotationAxis[0]+"\t"+pp.orbitRotationAxis[1]+"\t"+pp.orbitRotationAxis[2]+"\t"+pp.rotationAxis[0]+"\t"+pp.rotationAxis[1]+"\t"+pp.rotationAxis[2]);
		}
		
		System.out.println("End self-test of PlanetDataSource");
	}
    /**
     * public constant for consistent use in all other classes
     */
    public double daysPerYear=365.256363004;
    /** rotates a vector coordinates relating to the ecliptic plane such that the new coordinates relate to the earth equator plane */
    private Rotator incline = new Rotator(new double[] {1,0,0}, Math.toRadians(23.4392811));
    /** rotates a vector coordinates relating to the earth equator plane such that the new coordinates relate to the ecliptic plane */
    private Rotator inclineReversor= new Rotator(new double[] {1,0,0}, Math.toRadians(-23.4392811));

    /**  <pre>
     * Convert all planet data (provided in the String orbitData)
     * to the HashTable planetDataTable with a PlanetDataStructure for each planet.
     */
    PlanetDataSource() {
        this.readAllData();
    }


    /**  <pre>
     * Collection of all data of a planet to calculate its orbit and sky-view
     * method readAllData will create a Data class for each planet based on the text String in this PlanetData class
     * and put it in the public hashtable orbits
     * @author jacques
     *
     */
    public class PlanetDataStructure {
        String name;
        /** radius in km **/
        double radius;
        /** length of siderial day in days **/
        double siderialDay;
        /** rotation direction (clockwise CW or counterclockwise CCW **/
        String rotationDirection;
        /** aphelion time in millisecs from 1-1-1970 **/
        long aphelionTime;
        /** distance from sun to aphelion (farthest distance to sun) in km **/
        double aphelion;
        /** distance from sun to perihelion (closest distance to sun) in km **/
        double perihelion;
        /** orbital period in years **/
        double orbitalPeriod;
        /** orbital inclination in degrees **/
        double orbitalInclination;
        /** longitude of the ascending node (degrees) (where the planet rises through the ecliptic plane)**/
        double ascendingNode;
        /** argument of perihelion -- orbital angle between the ascending node and the perihelion - degrees **/
        double periHelionArgument;
        /** ORBIT rotation axis unit vector pointing roughly ecliptic North for CCW orbiting planets (all planets) and south for CW orbiting objects (maybe some meteors) */
        double[] orbitRotationAxis;
        /** rotation center **/
        String rotationCenter;
        /** orbits rotation direction (clockwise CW or counterclockwise CCW **/
        String orbitRotationDirection;
        /** prime meridian pointing to FirstPointOfARies: time in millisecs from 1-1-1970 **/
        long primeMeridianToFirstPointOfARies;
        /** right ascension of the North Pole of the rotation axis  [0, 360> degrees **/
        double raNorthPole;
        /** declination of the North Pole of the rotation axis [-90,90> degrees **/
        double declNorthPole;
        /** PLANET rotation axis unit vector pointing North for */
        double[] rotationAxis;
        /** class that does all orbit-calculations */
        PlanetOrbit po;
        /** position in the sky from a given viewpoint and a given time
         * skyLocation[0] = angle (degrees) of the two ground-projections of planet and the upper culmination of the sun, 0 for east; PI/2 for south; PI for west; 3PI/2 for North
         * skyLocation[1] = angle (degrees) between the ground and the vector pointing to the planet
         * skyLocation[3] = distance (km) between the observer and the planet
         * */
        double[] skyLocation = new double[3];
    }
    /**  <pre>
     * containing each planet orbitdata tagged by the planets name
     */
    public ArrayList<String> planetsList = new ArrayList<String>();
    public Hashtable<String,PlanetDataStructure> planetDataTable;

    /**  <pre>
     * Right ascension (abbreviated RA) is the angular distance
     * of a particular point measured eastward along the celestial equator (= earth equator)
     * First Point of Aries (Sun at the March equinox)
     * Declination's angle is measured north or south of the celestial equator
     *
     * @param xyzPlanet xyz coordinates of planet for which ra and decl should be determined
     * @param xyzEarth  xyz coordinates of earth
     * @return result[3]
     * result[0] is right ascension in (rad)
     * result[1] is declination (rad)
     * result[2] is the distance from earth (km)
     */
    double[] getRAandDeclination(double[] xyzPlanet, double[] xyzEarth) {
        double[] xyz = new double[3];
        for (int i = 0; i<3; i++) {
            xyz[i] = xyzPlanet[i] - xyzEarth[i];
        }
        xyz = incline.rotate(xyz);
        double[] result = new double[3];
        //right ascension
        result[0] = Math.atan2(xyz[1], xyz[0]);
        // declination
        result[1]  = Math.atan2(xyz[2], Math.sqrt(xyz[0]*xyz[0]+xyz[1]*xyz[1]));
        //result[1]  = Math.atan(xyzPlanet[2]/Math.sqrt(xyzPlanet[0]*xyzPlanet[0]+xyzPlanet[1]*xyzPlanet[1]));
        // distance
        result[2]  = Math.sqrt(xyz[0]*xyz[0]+xyz[1]*xyz[1]+xyz[2]*xyz[2]);
        return result;
    }

    double[] getXYZfromRaDecl(double ra, double decl) {
        double[] xyz = new double[3];
        xyz[0] = Math.cos(decl)*Math.cos(ra);
        xyz[1] = Math.cos(decl)*Math.sin(ra);
        xyz[2] = Math.sin(decl);
        double[] xyzR = inclineReversor.rotate(xyz);
        return xyzR;
    }

    private void readAllData() {
        planetDataTable = new Hashtable<String,PlanetDataStructure>();
        int iCountData = 0;
        String[] split = orbitData.split("\n");
        for (int i =0; i< split.length; i++) {
            if (split[i].contains("Planet.ID")) {
                iCountData++;
                String name = (split[i].split(" "))[1];
                PlanetDataStructure tData = new PlanetDataStructure();
                tData.name=name;
                for (int k =0; k< split.length; k++) {
                    if (split[k].startsWith(name)) {
                        String[] tmp = split[k].split(" ");
                        if (tmp[0].equals(name+".siderial.day")) {tData.siderialDay=Double.parseDouble(tmp[1]); iCountData++;}
                        if (tmp[0].equals(name+".aphelionTime")) {

                            String date = tmp[1];
                            String time = tmp[2];
                            String zone = tmp[3];
                            Calendar kal = Calendar.getInstance(TimeZone.getTimeZone(zone));
                            String[] dmy = date.split("-");
                            String[] hm = time.split(":");
                            kal.set(Integer.parseInt(dmy[2]), Integer.parseInt(dmy[1])-1, Integer.parseInt(dmy[0]),
                                    Integer.parseInt(hm[0]),Integer.parseInt(hm[1]));
                            tData.aphelionTime=kal.getTimeInMillis();
                            iCountData++;
                        }
                        if (tmp[0].equals(name+".Prime.meridian.to.FirstAries")) {

                            String date = tmp[1];
                            String time = tmp[2];
                            String zone = tmp[3];
                            Calendar kal = Calendar.getInstance(TimeZone.getTimeZone(zone));
                            String[] dmy = date.split("-");
                            String[] hm = time.split(":");
                            kal.set(Integer.parseInt(dmy[2]), Integer.parseInt(dmy[1])-1, Integer.parseInt(dmy[0]),
                                    Integer.parseInt(hm[0]),Integer.parseInt(hm[1]));
                            tData.primeMeridianToFirstPointOfARies=kal.getTimeInMillis();
                            iCountData++;
                        }
                        if (tmp[0].equals(name+".radius.km")) {tData.radius=Double.parseDouble(tmp[1]); iCountData++;}
                        if (tmp[0].equals(name+".aphelion.km")) {tData.aphelion=Double.parseDouble(tmp[1]); iCountData++;}
                        if (tmp[0].equals(name+".perihelion.km")) {tData.perihelion=Double.parseDouble(tmp[1]); iCountData++;}
                        if (tmp[0].equals(name+".orbital.period.year")) {tData.orbitalPeriod=Double.parseDouble(tmp[1]); iCountData++;}
                        if (tmp[0].equals(name+".inclination.ecliptic.degrees")) {tData.orbitalInclination=Double.parseDouble(tmp[1]); iCountData++;}
                        if (tmp[0].equals(name+".ascending.node.longitude.degrees")) {tData.ascendingNode=Double.parseDouble(tmp[1]); iCountData++;}
                        if (tmp[0].equals(name+".argument.perihelion.degrees")) {tData.periHelionArgument=Double.parseDouble(tmp[1]); iCountData++;}
                        if (tmp[0].equals(name+".orbital.rotation")) {

                            if (tmp[1].equals("counterclockwise")) {
                                tData.orbitRotationDirection="CCW";
                                iCountData++;
                            } else if (tmp[1].equals("clockwise")) {
                                tData.orbitRotationDirection="CW";
                                iCountData++;
                            } else {
                                System.err.println(split[k]+" must read counterclockwise or clockwise.");
                            }
                        }
                        if (tmp[0].equals(name+".rotating")) {

                            if (tmp[1].equals("counterclockwise")) {
                                tData.rotationDirection="CCW";
                                iCountData++;
                            } else if (tmp[1].equals("clockwise")) {
                                tData.rotationDirection="CW";
                                iCountData++;
                            } else {
                                System.err.println(split[k]+" must read counterclockwise or clockwise.");
                            }
                        }
                        if (tmp[0].equals(name+".rotation.center")) {
                            if (orbitData.contains("Planet.ID "+tmp[1])) {
                                tData.rotationCenter=tmp[1];
                                iCountData++;
                            } else {
                                System.err.println(split[k]+" is not a defined orbit-object.");
                            }
                        }
                        if (tmp[0].equals(name+".raNorthPole")) {tData.raNorthPole=Double.parseDouble(tmp[1]); iCountData++;}
                        if (tmp[0].equals(name+".declNorthPole")) {tData.declNorthPole=Double.parseDouble(tmp[1]); iCountData++;}
                    }
                    //System.out.println(i+" "+split[i]);
                }
                // tData postprocessing:
                // calculate vector normal to the orbit plane
                Rotator rotX = new Rotator(new double[] {0, 0,1}, Math.toRadians(tData.ascendingNode));
                double[] newX=rotX.rotate(new double[] {1,0,0});
                Rotator incliner = new Rotator(newX, Math.toRadians(tData.orbitalInclination));
                tData.orbitRotationAxis = incliner.rotate(new double[] {0,0,1});
                // calculate vector normal to the rotation plane
                tData.rotationAxis = getXYZfromRaDecl(Math.toRadians(tData.raNorthPole), Math.toRadians(tData.declNorthPole));
                tData.po = new PlanetOrbit(tData);
                // tData storage
                planetDataTable.put(name, tData);
                planetsList.add(name);
            }
            //System.out.println(i+" "+split[i]);
        }
        //System.out.println(iCountData+" data items read from "+split.length+" lines.");
        if (iCountData!=split.length) {
            System.err.println(iCountData+" data items read from "+split.length+" lines.");
            System.err.println("Each line should contain one datum.");
            System.out.println();
            System.out.println(planetDataTable.size());
            System.out.println(orbitData);
            System.exit(74248);
        }
    }

    // https://www.timeanddate.com/astronomy/perihelion-aphelion-solstice.html
    // All aphelion/perihelion times are in local Amsterdam time.
    // https://in-the-sky.org/news.php?id=20200327_11_101
    // https://in-the-sky.org/solarsystem.php
    // Other data from WIKI

    private String orbitData = ""
            + "Planet.ID Sun \n"
            + "Sun.siderial.day 25.05 \n" // at 16 degrees lat: the Sun's rotation varies with latitude
            + "Sun.rotating counterclockwise \n"
            + "Sun.radius.km 696342\n"
            + "Sun.rotation.center Sun \n"
            // Two different heliographic coordinate systems are used on the Sun.
            // The first is the Carrington heliographic coordinate system.
            // In this system, the prime meridian passes through the center of the solar disk as seen from the Earth on 9 November 1853, which is when the English astronomer Richard Christopher Carrington started his observations of sunspots.
            // The second is the Stonyhurst heliographic coordinates system, originated at Stonyhurst Observatory.
            // === prime meridian nog niet kunnen vinden ===
            + "Sun.Prime.meridian.to.FirstAries 20-3-2021 12:13 CEST \n"
            // https://en.wikipedia.org/wiki/Axial_tilt
            + "Sun.raNorthPole 286.13 \n"
            + "Sun.declNorthPole 63.87 \n"
            + "Sun.orbital.rotation counterclockwise \n" //niet gechecked


            + "Planet.ID Mercury \n"
            //23 Jun 2020 at 05:57 CEST (03:57 UTC)
            //+ "Mercury.aphelionTime 27-3-2020 12:00 CEST \n"
            + "Mercury.aphelionTime 23-6-2020 05:57 CEST \n"
            + "Mercury.siderial.day 58.646 \n"
            + "Mercury.rotating counterclockwise \n"
            + "Mercury.radius.km 4880 \n"
            + "Mercury.aphelion.km 69816900. \n"
            + "Mercury.perihelion.km 46001200. \n"
            + "Mercury.orbital.period.year 0.240846 \n"
            + "Mercury.inclination.ecliptic.degrees 7.005 \n"
            + "Mercury.ascending.node.longitude.degrees 45.331 \n"
            + "Mercury.argument.perihelion.degrees 29.124 \n"
            + "Mercury.orbital.rotation counterclockwise \n"
            + "Mercury.rotation.center Sun \n"
            //In 1975 the prime meridian of Mercury was defined[49][50] to be 20° east of the crater Hun Kal.
            // === prime meridian nog niet kunnen vinden ===
            + "Mercury.Prime.meridian.to.FirstAries 20-3-2021 12:13 CEST \n"
            // https://en.wikipedia.org/wiki/Axial_tilt
            + "Mercury.raNorthPole 281.01 \n"
            + "Mercury.declNorthPole 61.42 \n"

            + "Planet.ID Venus \n"
            + "Venus.aphelionTime 10-7-2020 14:43 UTC \n"
            + "Venus.siderial.day -243.025 \n"
            + "Venus.rotating clockwise \n" // retrogade
            + "Venus.radius.km 6051.8 \n"
            + "Venus.aphelion.km 108939000. \n"
            + "Venus.perihelion.km 107477000. \n"
            + "Venus.orbital.period.year 0.615198 \n"
            + "Venus.inclination.ecliptic.degrees 3.39458 \n"
            + "Venus.ascending.node.longitude.degrees 76.680 \n"
            + "Venus.argument.perihelion.degrees 54.884 \n"
            + "Venus.orbital.rotation counterclockwise \n" // probably, because explicit mentioned to be retrogade
            + "Venus.rotation.center Sun \n"
            //Defined[52] in 1992, the prime meridian of Venus passes through the central peak in the crater Ariadne.[53]
            // === prime meridian nog niet kunnen vinden ===
            + "Venus.Prime.meridian.to.FirstAries 20-3-2021 12:13 CEST \n"
            // https://en.wikipedia.org/wiki/Axial_tilt
            + "Venus.raNorthPole 272.76 \n"
            + "Venus.declNorthPole 67.16 \n"

            //rotation of earth when looking from space down to the North Pole: counterclockwise (because sunrise is in the east)
            //rotation of the earth around the sun when looking from space down to the North Pole: counterclockwise
            //   (because earth rotation is counterclockwise and the length of a stellar day (or siderial day) (23 hours, 56 minutes, 4.09 seconds) is shorter than 24 hours)
            + "Planet.ID Earth \n"
            + "Earth.aphelionTime 4-7-2020 13:34 CEST \n"
            + "Earth.siderial.day 0.9972696720084312 \n"
            + "Earth.rotating counterclockwise \n"
            // March equinox Sat, 20 Mar 2021 at 10:22 CEST
            // So prime meridian is about 12:00-10:22 = 1:38 away from pointing to FirstARies
            + "Earth.Prime.meridian.to.FirstAries 20-3-2021 12:13 CEST \n"
            // https://en.wikipedia.org/wiki/Axial_tilt
            + "Earth.raNorthPole 0 \n"
            + "Earth.declNorthPole 90 \n"
            + "Earth.radius.km 6371.0 \n"
            + "Earth.aphelion.km 152100000. \n"
            + "Earth.perihelion.km 147095000. \n"
            + "Earth.inclination.ecliptic.degrees 0.00005 \n"
            + "Earth.ascending.node.longitude.degrees -11.26064 \n"
            + "Earth.argument.perihelion.degrees 114.20783 \n"
            + "Earth.orbital.period.year 1.000 \n"
            // 365.256363004 days
            // public double daysPerYear=365.256363004;
            + "Earth.orbital.rotation counterclockwise \n"
            + "Earth.rotation.center Sun \n"

            + "Planet.ID Moon \n"
            //+ "Moon.aphelionTime 4-7-2020 13:34 CEST \n" +7
            //+ "Moon.aphelionTime 3-7-2020 13:34 CEST \n" +8
            //+ "Moon.aphelionTime 5-7-2020 13:34 CEST \n" +6
            //+ "Moon.aphelionTime 6-7-2020 13:34 CEST \n" +5
            //+ "Moon.aphelionTime 9-7-2020 13:34 CEST \n" +3
            //+ "Moon.aphelionTime 11-7-2020 13:34 CEST \n" +30 min
            //+ "Moon.aphelionTime 12-7-2020 13:34 CEST \n" -10min
            + "Moon.aphelionTime 2-7-2020 8:34 CEST \n" 
            + "Moon.siderial.day 27.321661 \n"
            + "Moon.rotating counterclockwise \n"
            + "Moon.radius.km 1737.4 \n"
            + "Moon.aphelion.km 405400. \n"
            + "Moon.perihelion.km 362600. \n"
            // Inclination	5.145° to the ecliptic
            + "Moon.inclination.ecliptic.degrees 5.145 \n"
            // Longitude of ascending node
            //    Regressing by one revolution in 18.61 years - this regression is not implemented in the orbital model... 
            + "Moon.ascending.node.longitude.degrees -11.26064 \n"
            // Argument of perigee
            // Progressing by one revolution in 8.85 years - this progression is not implemented in the orbital model...
            + "Moon.argument.perihelion.degrees 76. \n"
            // ----
            //+ "Moon.orbital.period.year 0.07397 \n"
            + "Moon.orbital.period.year 0.074801327964 \n"
            + "Moon.orbital.rotation counterclockwise \n"
            + "Moon.rotation.center Earth \n"
            // The prime meridian of the Moon lies directly in the middle of the face of the moon visible from Earth and passes near the crater Bruce.
            // === prime meridian nog niet kunnen vinden ===
            // === rough estimate using that earth is high in the air seen from the apollo landing sites
            + "Moon.Prime.meridian.to.FirstAries 27-3-2021 12:13 CEST \n"
            // https://en.wikipedia.org/wiki/Axial_tilt
            + "Moon.raNorthPole 270. \n"
            + "Moon.declNorthPole 66.54 \n"

            + "Planet.ID Mars \n"
            + "Mars.aphelionTime 13-7-2021 02:25 CEST \n"
            + "Mars.siderial.day 1.025957 \n"
            + "Mars.rotating counterclockwise \n"
            + "Mars.radius.km 3389.5 \n"
            + "Mars.aphelion.km 249200000. \n"
            + "Mars.perihelion.km 206700000. \n"
            + "Mars.inclination.ecliptic.degrees 1.85 \n"
            + "Mars.ascending.node.longitude.degrees 49.558 \n"
            + "Mars.argument.perihelion.degrees 286.502 \n"
            + "Mars.orbital.period.year 1.88082 \n"
            + "Mars.orbital.rotation counterclockwise \n"
            + "Mars.rotation.center Sun \n"
            // The prime meridian of Mars was established in 1971 and passes through the center of the crater Airy-0,
            // although it is fixed by the longitude of the Viking 1 lander, which is defined to be 47°.95137 west.
            // The Viking 1 lander touched down in western Chryse Planitia ("Golden Plain") at 22.697°N 312.05°E[2][11]
            // at a reference altitude of −2.69 kilometers (−1.67 mi) relative to a reference ellipsoid
            // with an equatorial radius of 3,397 kilometers (2,111 mi) and a flatness of 0.0105 (22.480° N, 47.967° W
            // planetographic) at July 20, 1976, 11:53:06 UTC (16:13 local Mars time).
            // Mars does not have time zones defined at regular intervals. Each lander so far has used an approximation of local solar time as its frame of reference.
            // Mars.Prime.meridian.to.FirstAries 20-3-2021 22:43 CEST produces Mars local time 4:04 PM at 47.967° W for UTC  11:53 20-7-1976
            + "Mars.Prime.meridian.to.FirstAries 20-3-2021 22:43 CEST \n"
            // https://en.wikipedia.org/wiki/Axial_tilt
            + "Mars.raNorthPole 317.68 \n"
            + "Mars.declNorthPole 52.89 \n"

            + "Planet.ID Jupiter \n"
            + "Jupiter.aphelionTime 28-12-2028 02:41 UTC \n"
            + "Jupiter.siderial.day 0.41354 \n"
            + "Jupiter.rotating counterclockwise \n"
            + "Jupiter.radius.km 69911 \n"
            + "Jupiter.aphelion.km 816620000. \n"
            + "Jupiter.perihelion.km 740520000. \n"
            + "Jupiter.inclination.ecliptic.degrees 1.303 \n"
            + "Jupiter.ascending.node.longitude.degrees 100.464 \n"
            + "Jupiter.argument.perihelion.degrees 273.867 \n"
            + "Jupiter.orbital.period.year 11.862 \n"
            + "Jupiter.orbital.rotation counterclockwise \n"
            + "Jupiter.rotation.center Sun \n"
            //  Jupiter has several coordinate systems because its cloud tops—the only part of
            // the planet visible from space—rotate at different rates depending on latitude.[56]
            // It is unknown whether Jupiter has any internal solid surface that would enable
            // a more Earth-like coordinate system.
            // System I and System II coordinates are based on atmospheric rotation,
            // and System III coordinates use Jupiter's magnetic field.
            // The prime meridians of Jupiter's four Galilean moons were established in 1979.[57]
            // === prime meridian nog niet kunnen vinden ===
            + "Jupiter.Prime.meridian.to.FirstAries 20-3-2021 12:13 CEST \n"
            // https://en.wikipedia.org/wiki/Axial_tilt
            + "Jupiter.raNorthPole 268.05 \n"
            + "Jupiter.declNorthPole 64.49 \n"

            + "Planet.ID Saturn \n"
            + "Saturn.aphelionTime 17-4-2018 13:31 CEST \n"
            + "Saturn.siderial.day 0.439 \n"
            + "Saturn.rotating counterclockwise \n"
            + "Saturn.radius.km 58232. \n"
            + "Saturn.aphelion.km 1514500000. \n"
            + "Saturn.perihelion.km 1352550000. \n"
            + "Saturn.inclination.ecliptic.degrees 2.458 \n"
            + "Saturn.ascending.node.longitude.degrees 113.665 \n"
            + "Saturn.argument.perihelion.degrees 339.392 \n"
            + "Saturn.orbital.period.year 29.4571 \n"
            + "Saturn.orbital.rotation counterclockwise \n"
            + "Saturn.rotation.center Sun \n"
            // Titan is the largest moon of Saturn and, like the Earth's moon, always has
            // the same face towards Saturn, and so the middle of that face is 0 longitude.
            // === prime meridian nog niet kunnen vinden ===
            + "Saturn.Prime.meridian.to.FirstAries 20-3-2021 12:13 CEST \n"
            // https://en.wikipedia.org/wiki/Axial_tilt
            + "Saturn.raNorthPole 40.60 \n"
            + "Saturn.declNorthPole 83.54 \n"

            + "Planet.ID Uranus \n"
            + "Uranus.aphelionTime 27-2-2009 01:22 CEST \n"
            + "Uranus.orbital.period.year 84.0205 \n"
            + "Uranus.rotating clockwise \n" //retrogade
            + "Uranus.aphelion.km 3008413179. \n"
            + "Uranus.perihelion.km 2742128967. \n"
            + "Uranus.inclination.ecliptic.degrees 0.773 \n"
            + "Uranus.ascending.node.longitude.degrees 74.006 \n"
            + "Uranus.argument.perihelion.degrees 96.998857 \n"
            + "Uranus.radius.km 25362. \n"
            + "Uranus.siderial.day 0.71833 \n"
            + "Uranus.orbital.rotation counterclockwise \n"
            + "Uranus.rotation.center Sun \n"
            // === prime meridian nog niet kunnen vinden ===
            + "Uranus.Prime.meridian.to.FirstAries 20-3-2021 12:13 CEST \n"
            // https://en.wikipedia.org/wiki/Axial_tilt
            + "Uranus.raNorthPole 257.43 \n"
            + "Uranus.declNorthPole -15.10 \n"

            + "Planet.ID Neptune \n"
            // Neptune at perihelion Neptune reaches its closest point to the
            // Sun. Wed, 03 Sep 2042 at 13:11 CEST (8050 days away) in Aries
            // Wolfrahm alfa: perihelion 16-7-2047
            // Wolrahm alpha: aphelion 10-12-2119
            // Estimated from inthesky.org: aphelion 24-12-2125
            + "Neptune.aphelionTime 24-12-2125 00:00 CEST \n"
            + "Neptune.orbital.period.year 164.8 \n"
            + "Neptune.aphelion.km 4537303418. \n"
            + "Neptune.perihelion.km 4459512525. \n"
            + "Neptune.inclination.ecliptic.degrees 1.767975 \n"
            + "Neptune.ascending.node.longitude.degrees 131.784 \n"
            + "Neptune.argument.perihelion.degrees 276.336 \n"
            + "Neptune.radius.km 24622. \n"
            + "Neptune.siderial.day 0.6713 \n"
            + "Neptune.rotating counterclockwise \n"
            + "Neptune.orbital.rotation counterclockwise \n"
            + "Neptune.rotation.center Sun \n"
            // Like Jupiter, Neptune is a gas giant, so any surface is obscured by clouds.
            // The prime meridian of its largest moon, Triton, was established in 1991.[58]
            // === prime meridian nog niet kunnen vinden ===
            + "Neptune.Prime.meridian.to.FirstAries 20-3-2021 12:13 CEST \n"
            // https://en.wikipedia.org/wiki/Axial_tilt
            + "Neptune.raNorthPole 299.36 \n"
            + "Neptune.declNorthPole 43.46 \n"

            + "Planet.ID Pluto \n"
            + "Pluto.aphelionTime 1-2-2114 00:00 CEST \n"
            + "Pluto.siderial.day 6.387230 \n"
            + "Pluto.rotating clockwise \n" //retrogade
            + "Pluto.radius.km 2376.6 \n"
            + "Pluto.aphelion.km 7375930000. \n"
            + "Pluto.perihelion.km 4436820000. \n"
            + "Pluto.inclination.ecliptic.degrees 17.10 \n"
            + "Pluto.ascending.node.longitude.degrees 110.299 \n"
            + "Pluto.argument.perihelion.degrees 113.834 \n"
            + "Pluto.orbital.period.year 247.94\n"
            + "Pluto.orbital.rotation counterclockwise \n"
            + "Pluto.rotation.center Sun \n"
            // Pluto's prime meridian is defined as the center of the face that is
            // always pointed towards Charon, its largest moon, as the two are tidally locked towards each other.
            // === prime meridian nog niet kunnen vinden ===
            + "Pluto.Prime.meridian.to.FirstAries 20-3-2021 12:13 CEST \n"
            // https://en.wikipedia.org/wiki/Axial_tilt
            + "Pluto.raNorthPole 312.99 \n"
            + "Pluto.declNorthPole 6.16 \n"
            
            // The fictional Olympus_Space_Station's is located high up in the ecliptic northpole
            + "Planet.ID Olympus_Space_Station \n"
            + "Olympus_Space_Station.aphelionTime 1-2-2502114 00:00 CEST \n"
            + "Olympus_Space_Station.siderial.day 1e50 \n"
            + "Olympus_Space_Station.rotating counterclockwise \n" 
            + "Olympus_Space_Station.radius.km 1 \n"
            + "Olympus_Space_Station.aphelion.km 20000000000. \n"
            + "Olympus_Space_Station.perihelion.km 20000000000. \n"
            + "Olympus_Space_Station.inclination.ecliptic.degrees 90.0 \n"
            + "Olympus_Space_Station.ascending.node.longitude.degrees 0. \n"
            + "Olympus_Space_Station.argument.perihelion.degrees 0. \n"
            + "Olympus_Space_Station.orbital.period.year 10000000 \n"
            + "Olympus_Space_Station.orbital.rotation counterclockwise \n"
            + "Olympus_Space_Station.rotation.center Sun \n"
            + "Olympus_Space_Station.Prime.meridian.to.FirstAries 31-8-10002022 12:13 CEST \n"
            // https://en.wikipedia.org/wiki/Axial_tilt
            + "Olympus_Space_Station.raNorthPole 0 \n"
            + "Olympus_Space_Station.declNorthPole 0.01 \n"

            // The fictional Aries_Space_Station's is located in the direction of First Point of Aries (Vernal equinox)
            + "Planet.ID Aries_Space_Station \n"
            + "Aries_Space_Station.aphelionTime 1-2-2022 00:00 CEST \n"
            + "Aries_Space_Station.siderial.day 1e50 \n"
            + "Aries_Space_Station.rotating counterclockwise \n" 
            + "Aries_Space_Station.radius.km 1 \n"
            + "Aries_Space_Station.aphelion.km 30000000000. \n"
            + "Aries_Space_Station.perihelion.km 30000000000. \n"
            + "Aries_Space_Station.inclination.ecliptic.degrees 0.0 \n"
            + "Aries_Space_Station.ascending.node.longitude.degrees 0. \n"
            + "Aries_Space_Station.argument.perihelion.degrees 180. \n"
            + "Aries_Space_Station.orbital.period.year 10000000 \n"
            + "Aries_Space_Station.orbital.rotation counterclockwise \n"
            + "Aries_Space_Station.rotation.center Sun \n"
            + "Aries_Space_Station.Prime.meridian.to.FirstAries 31-8-10002022 12:13 CEST \n"
            // https://en.wikipedia.org/wiki/Axial_tilt
            + "Aries_Space_Station.raNorthPole 0 \n"
            + "Aries_Space_Station.declNorthPole 0.01 \n"

            // The fictional Sagittarii_Space_Station's is located constellation of Sagittarius ner Mu Sagittarii. 
            + "Planet.ID Sagittarii_Space_Station \n"
            + "Sagittarii_Space_Station.aphelionTime 1-2-2022 00:00 CEST \n"
            + "Sagittarii_Space_Station.siderial.day 1e50 \n"
            + "Sagittarii_Space_Station.rotating counterclockwise \n" 
            + "Sagittarii_Space_Station.radius.km 1 \n"
            + "Sagittarii_Space_Station.aphelion.km 30000000000. \n"
            + "Sagittarii_Space_Station.perihelion.km 30000000000. \n"
            + "Sagittarii_Space_Station.inclination.ecliptic.degrees 0.0 \n"
            + "Sagittarii_Space_Station.ascending.node.longitude.degrees 0. \n"
            + "Sagittarii_Space_Station.argument.perihelion.degrees 90. \n"
            + "Sagittarii_Space_Station.orbital.period.year 10000000 \n"
            + "Sagittarii_Space_Station.orbital.rotation counterclockwise \n"
            + "Sagittarii_Space_Station.rotation.center Sun \n"
            + "Sagittarii_Space_Station.Prime.meridian.to.FirstAries 31-8-10002022 12:13 CEST \n"
            // https://en.wikipedia.org/wiki/Axial_tilt
            + "Sagittarii_Space_Station.raNorthPole 0 \n"
            + "Sagittarii_Space_Station.declNorthPole 0.01 \n"

            + "";
}
