package com.jacquesb.planetario_a;

import java.util.Calendar;
import java.util.TimeZone;

public class PlanetOrbit {
	String name;
	double orbitalPeriod; //year
	double aphelion; // farthest distance (km) of the orbit to the sun
	double perihelion; // smallest distance (km) of the orbit to the sun
	long aphelionTime; // time in milliseconds since epoch at which the planet is at its farthest point (aphelion)

	PlanetDataSource.PlanetDataStructure data;
	/**  <pre>
	 * return XYZ coordinates of a planet or other orbiting object
	 * origin is the rotation center (sun or planet if the orbiting object is a moon)
	 * @param timeInMillis timeInMillis produced by java.Calendar
	 * @return result[3]
	 * result[0] x coordinate (km); positive X-axis points to the First Point of Aries;
	 * result[1] y coordinate (km); positive Y-axis is 90 degrees CCW to the positive X-axis seen from ecliptic North;
	 * result[2] z coordinate (km); positive Z-axis is pointing to the ecliptic North;
	 */
	public double[] getXYZcoords(long timeInMillis) {
		if (data.name.equals(data.rotationCenter)) {
			double[] xyz = {0., 0., 0.};
			return xyz;
		}
		double[] result = elliptical_periaps_sector_coords(timeInMillis);
		double[] resultz1 = z1.rotate(new double[] {result[2], result[3], 0.});
		double[] resultx2 = x2.rotate(resultz1);
		double[] resultz3 = z3.rotate(resultx2);
		return resultz3;
	}

	/**  <pre>
	 * return angle of a given ellipse sector area defined by a line
	 * from a pole to to its apoapse (farthest point on the ellipse along the major axis)
	 * and a point P on the ellipse
	 * @param timeInMillis timeInMillis produced by java.Calendar
	 * @return result[4]
	 * result[0] theta angle (rad) between the major axis and the line to P;
	 * result[1] r distance between pole and point P;
	 * result[2] x coordinate of point P;
	 * result[3] y coordinate of point P;
	 */
	public double[] elliptical_periaps_sector_coords(long timeInMillis) {
		double millisPerYear = 1000.*3600.*24.*365.256363004;
		//  or? double millisPerYear = 1000.*3600.*24.*365.256363004/1.00001742096;
		long delta = timeInMillis-aphelionTime;
		long orbitInMillis = (long)(millisPerYear*orbitalPeriod);
		long otime = floorMod(delta, orbitInMillis);
		double s_area = Math.PI*a*b*(double)otime/(millisPerYear*orbitalPeriod);
		double result[] = elliptical_periaps_sector_coords(s_area);
		return result;
	}

	/**  <pre>
	 * Math.floormod workaround for old android versions
	 * NOTE:
 	 *  in android code: if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.N) {
 	 *  in java code:    if (0 == 0) {
	 * @param x numerator
	 * @param y denominator
	 * @return the floor modulus of x over y
	 */
	Long floorMod(long x, long y) {
		//if (0 == 0) {
		if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.N) {
			long fm = Math.floorMod(x,y);
			return fm;
		}

		// The floor modulus fm is x - (floorDiv(x, y) * y),
		// has the same sign as the divisor y,
		// and is in the range of -abs(y) < fm < +abs(y).

		long fm = x;
		if (x == 0) return (long)0;
		if (y>0) {
			if (x > 0) {
				// for positive x and y:
				while (fm <= 0) fm = fm - y;
				fm = fm + y;
			} else {
				// for negative x and positive y:
				while (fm <= 0) fm = fm + y;
			}
		} else {
			if (x > 0) {
				// for positive x and negative y:
				while (fm >= 0) fm = fm + y;
			} else {
				// for negative x and negative y:
				while (fm <= 0) fm = fm - y;
				fm = fm + y;
			}
		}
		return fm;
	}


	PlanetOrbit(PlanetDataSource.PlanetDataStructure data) {
		this.data = data;
		this.aphelionTime=data.aphelionTime;
		this.name = data.name;
		this.orbitalPeriod=data.orbitalPeriod;
		this.aphelion=data.aphelion;
		this.perihelion=data.perihelion;
		if (! data.name.equals(data.rotationCenter)) {
			initOrbitData();
			initRotations();
		}
	}
	//Rotation z1, x2, z3;
	Rotator z1, x2, z3;
	/**  <pre>
	 * https://en.wikipedia.org/wiki/Longitude_of_the_ascending_node
	 *
	 */
	void initRotations() {
		// rotate around the Z-axis (perpendicular to the XY-orbit plane)
		// until the new positive X-axis and the longitude of the ascending node overlap
		// the perihelion is before rotation) on the negative X-axis
		// without rotation the CCW angle between the perihelion and the positive X axis is 180 degrees
		// after the rotation it must be data.argument.perihelion.degrees
		// e.g. if data.argument.perihelion.degrees = 190 degrees, the ellipse must rotate
		// 10 degrees counterclockwise.
		//z1 = new Rotation("Z",Math.toRadians(data.periHelionArgument-180.));
		//z1 = new Rotation("Z",Math.toRadians(data.periHelionArgument-180.));
		z1 = new Rotator(new double[] {0,  0, 1},Math.toRadians(data.periHelionArgument-180.));
		// rotate along the ascending node (postive X axis)
		// such that (for small inclination, < 180 degrees) positive Y-values get positive Z-values
		// x2 = new Rotation("X",Math.toRadians(data.orbitalInclination));
		x2 = new Rotator(new double[] {1, 0,0},Math.toRadians(data.orbitalInclination));
		// the positive X-axis is now pointing to the ascending node
		// after the rotation around the Z-axis the postive X-axis must point to  the First Point of Aries
		// E.g. if the longitude of the ascending node is 10 degrees, the orbital ellipse
		// must rotate 10 degrees counterclockwise, which
		// is the same as if the X-axis rotates 10 degrees clockwise.
		//z3 = new Rotation("Z",Math.toRadians(data.ascendingNode));
		z3 = new Rotator(new double[] {0, 0, 1},Math.toRadians(data.ascendingNode));
	}
	private void initOrbitData() {
		a = (aphelion+perihelion)/2.;
		double ab = a - perihelion;
		b = Math.sqrt(a*a-ab*ab);
		// divide the ellipse in 64 (must be 2**n) sectors of equal area
		nFourier = 64;
		double sector_area=Math.PI*a*b/nFourier;
		double[] angle = new double[nFourier];
		//System.out.println("area       angle");
		angle[0]=0;
		for (int sector=1; sector<nFourier; sector++) {
			double angleUp=2.*Math.PI;
			double angleDown = 0;
			boolean found = false;
			while (! found) {
				double s=sector*sector_area;
				double test = (angleUp+angleDown)/2.;
				double f = this.elliptical_periaps_sector_area(a, b, test);
				if (f>s) {
					angleUp = test;
				} else {
					angleDown = test;
				}
				if (Math.abs(f-s)/(f+s)<1e-4) {
					found = true;
					angle[sector]=test;
					//System.out.println(""+s+" "+test);
				}
			}
		}
		// prepare FFT
		double[] inputReal = new double[nFourier];
		double[] inputImag = new double[nFourier];
		for (int d = 0; d<nFourier; d++) {
			inputReal[d] = angle[d]-2.*Math.PI*d/nFourier;
			inputImag[d] = 0;
			//System.out.println(d+" "+inputReal[d]);
		}
		double[] fft= FFTbase.fft(inputReal, inputImag, true);
		// angle(sector_area) consists of a sum of sines, so fft[odd] = 0; fft[even]= coefficient

		for (int i = 0; i < fft.length; i++) {
			//System.out.println(i+" "+fft[i]);
		}

		double sum = 0;
		for (int i = 0; i < nFourier/4; i++) {
			sum = sum+fft[i*2+1];
			//System.out.println(i+" "+sum);
		}
		int p=0;
		double sumP=0;
		for (int i = 0; i < nFourier/4; i++) {
			sumP = sumP+fft[i*2+1];
			if (Math.abs(sumP-sum)/Math.abs(sumP+sum)>0.001) p = i;
			//System.out.println(i+" "+sum);
		}
		ifft = new double[p];
		for (int i = 0; i < p; i++) {
			ifft[i] = fft[(i+1)*2+1];
			//System.out.println(i+" "+ifft[i]);
		}
	}
	private int nFourier;
	private double[] ifft;
	private double a;
	private double b;
	/**  <pre>
	 * return angle of a given ellipse sector area defined by a line
	 * from a pole to to its apoapse (farthest point on the ellipse along the major axis)
	 * and a point P on the ellipse
	 * @param area of the ellipse sector defined by pole, apoapse and a point P
	 * @return result[4]
	 * result[0] theta angle (rad) between the major axis and the line to P;
	 * result[1] r distance between pole and point P;
	 * result[2] x coordinate of point P;
	 * result[3] y coordinate of point P;
	 */
	public double[] elliptical_periaps_sector_coords(double area) {
		double[] result = new double[4];
		double r_area = area/(Math.PI*a*b);
		double theta = 0;
		//System.out.println("FFT: "+ifft.length);
		for (int k = 0; k < ifft.length; k++) {
			theta = theta + ifft[k]*Math.sin((k+1)*2.*Math.PI*r_area);
			//System.out.println(""+(k+1)+" "+(2*ifft[k]/Math.sqrt(nFourier)));
			//f = f + fft[2*k+1]*Math.sin(k*2.*Math.PI*i/n);
			//f = f*2./Math.sqrt(n);
		}
		theta = 2.*theta/Math.sqrt(nFourier)+2.*area/(a*b);
		result[0]=theta;
		double excentricity = Math.sqrt(1.-b*b/(a*a));
		// define X-axis as the major axis
		// origin is the left pole
		// distance between leftPole and P is:
		double rLP = a*(1-excentricity*excentricity)/(1.-excentricity*Math.cos(theta));
		result[1]=rLP;
		// x- and y-coordinates of P:
		double xP = rLP*Math.cos(theta);
		double yP = rLP*Math.sin(theta);
		result[2]=xP;
		result[3]=yP;
		return result;
	}

	/** <pre>
	 * return area in the ellipse sector area defined by a line
	 * from a pole to to its apoapse (farthest point on the ellipse along the major axis)
	 * and a point P on the ellipse
	 * @param a half length of the major axis
	 * @param b half length of the minor axis
	 * @param theta angle (rad) between the major axis and the line to P
	 * @return area of the ellipse sector
	 */
	public double elliptical_periaps_sector_area(double a, double b, double theta) {
		double excentricity = Math.sqrt(1.-b*b/(a*a));
		// define X-axis as the major axis
		// pole is the left pole
		// x-coordinate of the left pole is (negative):
		double xLeftPole = -Math.sqrt(a*a-b*b);
		// distance between leftPole and P is:
		double rLP = a*(1-excentricity*excentricity)/(1.-excentricity*Math.cos(theta));
		// x- and y-coordinates of P:
		double xP = rLP*Math.cos(theta)+xLeftPole;
		double yP = rLP*Math.sin(theta);
		// angle major axis - center-P
		double thetaC = atan3(yP, xP);
		double area1 = this.elliptical_center_sector_area(a, b, thetaC);
		// area of the triangle leftpole-center-P
		double area2 = 0.5*Math.sqrt(a*a-b*b)*yP;
		return area1+area2;
	}
	/**
	 * Returns the angle theta from the conversion of rectangular coordinates (x, y) to polar coordinates (r, theta).
	 * This method computes the phase theta by computing an arc tangent of y/x in the range of to 2*pi.
	 * @param y y coordinate of a point P
	 * @param x x coordinate of a point P
	 * @return theta angle (rad) between positive X-axis and line origin-P
	 */
	private double atan3(double y, double x) {
		double theta =  Math.atan2(y, x);
		if (theta < 0) theta = theta+2.*Math.PI;
		return theta;
	}
	/**
	 * return area in the ellipse sector area defined by a line
	 * from a center to its apoapse (farthest point on the ellipse along the major axis)
	 * and a point P on the ellipse
	 * @param a half length of the major axis
	 * @param b half length of the minor axis
	 * @param theta angle (rad) between the major axis and the line to P
	 * @return area of the ellipse sector
	 */
	public double elliptical_center_sector_area(double a, double b, double theta) {
		double f = (a*b/2.)*(theta - Math.atan((b-a)*Math.sin(2.*theta)/(b+a+(b-a)*Math.cos(2.*theta))));
		return f;
	}


}
