package com.jacquesb.planetario_a;

/** <pre>
 *
 * ============================
 * roatations using quaternions
 * ============================
 *
 * rotate X = (x1, x2, x3) around unit-vector v = (v1, v2, v3) by angle theta
 *
 * quarterion qX = 0 + i x1 + j x2 + k x3
 *
 * quarternion h = a + i b + j c + k d
 * quarternion h* = a - i b - j c - k d
 *
 * a = cos(theta/2)
 * b = v1 sin(theta/2)
 * c = v2 sin(theta/2)
 * d = v3 sin(theta/2)
 *
 * Xrotated = h p h*
 *
 * multiplication rules:
 *
 * i i = j j = k k = i j k = -1
 * and:
 * ij = k, ji = -k
 * jk = i, kj = -i
 * ki = j, ik = -j
 *
 * Multiplication of r and p:
 * (r1 + i r2 + j r3 + k r4) * (p1 + i p2 + j p3 + k p4) =
 *     r1 p1 + i r1 p2 + j r1 p3 + k r1 p4 +
 * + i r2 p1 -   r2 p2 + k r2 p3 - j r2 p4 +
 * + j r3 p1 - k r3 p2 -   r3 p3 + i r3 p4 +
 * + k r4 p1 + j r4 p2 - i r4 p3 -   r4 k4 =
 * q1 + i q2 + j q3 + k q4
 *
 * met:
 *   q1 =    r1 p1 - r2 p2 - r3 p3 - r4 p4
 * i q2 = i (r1 p2 + r2 p1 + r3 p4 - r4 p3)
 * j q3 = j (r1 p3 - r2 p4 + r3 p1 + r4 p2)
 * k q4 = k (r1 p4 + r2 p3 - r3 p2 + r4 p1)
 * </pre>
 *
 * @author jacques
 *
 */
public class Rotator {
	/**  <pre>
	 * Instellen van rotator zodat vector1 met hoek angle richting vector2 draait 
	 * @param vector1
	 * @param vector2
	 * @param angle    draaihoek in graden
	 */
	public Rotator(double[] vector1, double[] vector2, double angle) {
		setRotation(vector1, vector2, angle);
	}
	/**  <pre>
	 * Instellen van rotator zodat vector1 met hoek angle naar vector2 draait
	 * @param vector1
	 * @param vector2
	 * @param angle    draaihoek in graden
	 */
	public void setRotation(double[] vector1, double[] vector2, double angle) {
		double[] r = crossproduct(vector2, vector1);
		//double angle = angle(vector2, vector1);
		while (angle > 360) angle = angle-360;
		while (angle < 0) angle = angle+360;
		if (r[0]==0&&r[1]==0&&r[2]==0) {
			if (angle == 0) {
				setRotation(vector1, 0);
			} else if (angle == 180) {
				r = crossproduct(new double[] {vector1[0]+1, vector1[1]-1,vector1[2]}, vector1);
				setRotation(r, -Math.toRadians(angle));
			} else {
				System.err.println("rotator undefined");
				if ((angle < 90)||(angle > 270)) {
					System.err.println("rotator set to do nothing");
					setRotation(vector1, 0);
				} else {
					System.err.println("rotator set to rotate 180 degrees");
					r = crossproduct(new double[] {vector1[0]+1, vector1[1]-1,vector1[2]}, vector1);
					setRotation(r, Math.toRadians(180));
				}
			}
		} else {
			setRotation(r, -Math.toRadians(angle));
		}
	}
	/**  <pre>
	 * Instellen van rotator zodat vector1 naar vector2 draait 
	 * @param vector1
	 * @param vector2
	 */
	public Rotator(double[] vector1, double[] vector2) {
		setRotation(vector1, vector2);
	}
	/**  <pre>
	 * Instellen van rotator zodat vector1 naar vector2 draait
	 * @param vector1
	 * @param vector2
	 */
	public void setRotation(double[] vector1, double[] vector2) {
		double[] r = crossproduct(vector2, vector1);
		double angle = angle(vector2, vector1);
		if (angle == 0) {
			setRotation(vector1, -angle);
		} else if(Math.toDegrees(angle)==180) {
			// find a vector that is pointing in another direction than vector1:
			double[] vectorA = {1,0,0};
			r = crossproduct(vectorA, vector1);
			if (r[0]*r[0]+r[1]*r[1]+r[2]*r[2]==0) {
				double[] vectorB = {0,1,0};
				r = crossproduct(vectorB, vector1);
			}
			if (r[0]*r[0]+r[1]*r[1]+r[2]*r[2]==0) {
				double[] vectorC = {0,0,1};
				r = crossproduct(vectorC, vector1);
			}
			setRotation(r, -angle);
		} else {
			setRotation(r, -angle);
		}

	}
	/**  <pre>
	 * Instellen van de rotatie-as en de rotatiehoek
	 * - als de rotatie-as-vector naar je toewijst, wordt tegen de klok in (CCW) gedraaid
	 *   (in een normaal georienteerd XYZ systeem).
	 * @param v		double[3] met x, y, en z coordinaat van de rotatieas-vector
	 * @param angle	angle (rad) draaihoek
	 */
	public Rotator(double[] v, double angle) {
		setRotation(v, angle);
	}
	/**  <pre>
	 * Instellen van de rotatie-as en de rotatiehoek
	 * - als de rotatie-as-vector naar je toewijst, wordt tegen de klok in (CCW) gedraaid
	 *   (in een normaal georienteerd XYZ systeem).
	 * @param v		double[3] met x, y, en z coordinaat van de rotatieas-vector
	 * @param angle	angle (rad) draaihoek
	 */
	public void setRotation(double[] v, double angle) {
		if (v.length!=3) {
			System.err.println("Invalid dimension v "+v.length);
			System.exit(8672332);
		}
		double n = Math.sqrt(Math.pow(v[0],2)+Math.pow(v[1],2)+Math.pow(v[2],2));
		h = new double[4];
		hStar = new double[4];

		h[0] = Math.cos(angle/2.);
		h[1] = v[0] * Math.sin(angle/2.)/n;
		h[2] = v[1] * Math.sin(angle/2.)/n;
		h[3] = v[2] * Math.sin(angle/2.)/n;
		hStar[0] = h[0];
		hStar[1] = -h[1];
		hStar[2] = -h[2];
		hStar[3] = -h[3];
	}
	/**  <pre>
	 * Roteren van vector p volgens de met setRotation ingestelde draai-as en hoek
	 * - als de rotatie-as-vector naar je toewijst, wordt p tegen de klok in (CCW) gedraaid
	 *   (in een normaal georienteerd XYZ systeem).
	 * @param p	double[3] met x, y, en z coordinaat van het punt p
	 * @return	double[3] met x, y, en z coordinaat van het geroteerde punt p
	 */
	public double[] rotate(double[] p) {
		if (p.length!=3) {
			System.err.println("Invalid dimension p "+p.length);
			System.exit(8672143);
		}
		double[] q = {0, p[0], p[1], p[2]};
		double[] t = multiply(h, q);
		double[] s = multiply(t,hStar);
		double[] result = new double[3];
		result[0] = s[1];
		result[1] = s[2];
		result[2] = s[3];
		return result;
	}
	/**  <pre>
	 * The cross product or vector product a x b 
	 *   (a binary operation on two vectors in a three-dimensional oriented Euclidean vector space) 
	 * @param a
	 * @param b
	 * @return a x b
	 */
	double[] crossproduct(double[] a, double[] b) {
		double[] s = new double[3];
		s[0] = a[1]*b[2] - a[2]*b[1];
		s[1] = a[2]*b[0] - a[0]*b[2];
		s[2] = a[0]*b[1] - a[1]*b[0];
		return s;
	}
	/**  <pre>
	 * get the angle theta between two vectors a and b 
	 *    by using the dotproduct a.b = |a| |b| cos(theta)
	 *    and the length of the crossproduct |a x b| =   |a| |b| sin(theta)
	 * @param a 3D vector
	 * @param b 3D vector
	 * @return angle between a and b: [0,PI> rads
	 */
	private double angle(double[] a, double[] b) {
		double dotproduct = a[0]*b[0]+a[1]*b[1]+a[2]*b[2];
		double lengtha = Math.sqrt(a[0]*a[0]+a[1]*a[1]+a[2]*a[2]);
		double lengthb = Math.sqrt(b[0]*b[0]+b[1]*b[1]+b[2]*b[2]);
		double cos_angle = dotproduct/(lengtha*lengthb);
		double angle = Math.acos(cos_angle);
		return angle;
	}


	// define rotation quaternions:
	double[] h, hStar;

	private double[] multiply(double[] r, double[] p) {
		double[] q = new double[4];
		q[0] = r[0]*p[0] - r[1]*p[1] - r[2]*p[2] - r[3]*p[3];
		q[1] = r[0]*p[1] + r[1]*p[0] + r[2]*p[3] - r[3]*p[2];
		q[2] = r[0]*p[2] - r[1]*p[3] + r[2]*p[0] + r[3]*p[1];
		q[3] = r[0]*p[3] + r[1]*p[2] - r[2]*p[1] + r[3]*p[0];
		return q;
	}
	void printQ(double[] q) {
		String qp = ""+q[0];
		for (int i = 1; i < q.length; i++) {
			qp = qp + ", "+q[i];
		}
		System.out.println(qp);
	}
	/**  <pre>
	 * testruns van dit rotatie-algoritme
	 * @param args	worden niet gebruikt
	 */
	public static void main(String[] args) {
		System.out.println("====================");
		System.out.println("Self-test of Rotator");
		System.out.println("====================");

		System.out.println("\nRotate (1,0,0) 45 degrees around Z-axis");

		double[] v = new double[3];
		v[0] = 0;
		v[1] = 0;
		v[2] = 1;
		double angle = Math.toRadians(45.);

		double[] p = new double[3];
		p[0] = 1;
		p[1] = 0;
		p[2] = 0;

		Rotator rotate = new Rotator(v, angle);
		double[] r = rotate.rotate(p);
		rotate.printQ(r);

		System.out.println("\nRotate (1,0,0) 45 degrees around Y-axis");

		v[0] = 0;
		v[1] = 2;
		v[2] = 0;

		rotate.setRotation(v, angle);
		r = rotate.rotate(p);
		rotate.printQ(r);

		v[0] = 1;
		v[1] = 0;
		v[2] = 0;

		System.out.println("\nRotate (1,0,1) 45 degrees around X-axis");

		p[2] = 1;
		rotate.setRotation(v, angle);
		r = rotate.rotate(p);
		rotate.printQ(r);

		System.out.println("\nRotate (1,0,0) two times");
		p[2] = 0;
		v[0] = 0;
		v[1] = 0;
		v[2] = 1;

		rotate.setRotation(v, angle);
		r = rotate.rotate(p);
		rotate.setRotation(r, angle);
		r = rotate.rotate(p);
		rotate.printQ(r);

		System.out.println("\nCrossproduct X x Y = (1,0,0) x (0,1,0)");
		double[] x = {1,0,0};
		double[] y = {0,1,0};
		double[] xy = rotate.crossproduct(x, y);
		rotate.printQ(xy);

		System.out.println("\nCrossproduct Y x X = (0,1,0) x (1,0,0)");
		double[] yx = rotate.crossproduct(y, x);
		rotate.printQ(yx);

		System.out.println("\nAngle between X and Y");
		System.out.println(""+rotate.angle(x,y));

		System.out.println("\nAngle between X and -Y");
		double[] yMin = {0,-1,0};
		System.out.println(""+rotate.angle(x,yMin));

		System.out.println("\nRotate (1,0,0) to (0,1,0)");
		rotate.setRotation(x, y);
		r = rotate.rotate(x);
		rotate.printQ(r);

		System.out.println("\nRotate (1,0,0) to (0,-1,0)");
		rotate.setRotation(x, yMin);
		r = rotate.rotate(x);
		rotate.printQ(r);
		System.out.println("rotate (0,1,0)");
		r = rotate.rotate(y);
		rotate.printQ(r);
		System.out.println("rotate (0,0,1)");
		double[] z = {0,0,1};
		r = rotate.rotate(z);
		rotate.printQ(r);

		System.out.println("\nRotate (0,0,1) to (0,-1,0)");
		rotate.setRotation(z, yMin);
		r = rotate.rotate(z);
		rotate.printQ(r);
		System.out.println("rotate (0,1,0)");
		r = rotate.rotate(y);
		rotate.printQ(r);
		System.out.println("rotate (0,-1,0)");
		r = rotate.rotate(yMin);
		rotate.printQ(r);
		System.out.println("rotate (1,0,0)");
		r = rotate.rotate(x);
		rotate.printQ(r);

		System.out.println("\nRotate (1,0,0) to (0,1,0) over 45 graden");
		rotate.setRotation(x, y, 45);
		r = rotate.rotate(x);
		rotate.printQ(r);
		System.out.println("\nRotate (1,0,0) to (0,1,0) over -45 graden");
		rotate.setRotation(x, y, -45);
		r = rotate.rotate(x);
		rotate.printQ(r);
		System.out.println("\nRotate (1,0,0) to (0,1,0) over -90 graden");
		rotate.setRotation(z, y, -90);
		r = rotate.rotate(z);
		rotate.printQ(r);
		System.out.println("\nRotate (1,0,0) to (1,0,0)");
		rotate.setRotation(x, x);
		r = rotate.rotate(x);
		rotate.printQ(r);
		System.out.println("\nRotate (-1,0,0) to (1,0,0)");
		double[] minX = {-1,0,0};
		rotate.setRotation(minX, x);
		r = rotate.rotate(minX);
		rotate.printQ(r);
		System.out.println("\nRotate (0,0,1) to (0,0,-1)");
		rotate.setRotation(z, new double[] {0,0,-1});
		r = rotate.rotate(z);
		rotate.printQ(r);
		System.out.println("\nRotate (1,0,0) to (-1,0,0) over 2 graden");
		rotate.setRotation(x, new double[] {-1,0,0}, 2);
		r = rotate.rotate(x);
		rotate.printQ(r);
		System.out.println("\nRotate (1,0,0) to (-1,0,0) over -179 graden");
		rotate.setRotation(x, new double[] {-1,0,0}, -179);
		r = rotate.rotate(x);
		rotate.printQ(r);
	}

}
